#include "stdio.h"
#include "string.h"
#include "stdlib.h"

#include "satwrite.h"
#include "miemath.h"

#define  COMPRESO(x,y,z) (((x>=y) && (x<=z)) ? 1 : 0)

const double
  pi       = 3.1415926535897932385,   //PiGreco
  twopi    = 6.28318530717958623, //2*PiGreco
  sr       = 696000.0,        //Solar radius - kilometers (IAU 76)
  AU       = 1.49597870E8,    //Astronomical unit - kilometers (IAU 76)

  ae       = 1,
  tothrd   =  .6666666666666666667,
  xkmper   = 6378.135,        //{Earth equatorial radius - kilometers (WGS '72)}
  f        = 0.003352779454167505,//1/298.26, //{Earth flattening (WGS '72)}
  ge       = 398600.8,        //{Earth gravitational constant (WGS '72)}
  J2       = 1.0826158E-3,    //{J2 harmonic (WGS '72)}
  J3       = -2.53881E-6,     //{J3 harmonic (WGS '72)}
  J4       = -1.65597E-6,     //{J4 harmonic (WGS '72)}
  ck2      = 1.0826158E-3/2,
  ck4      = 6.2098875E-7,    //-3*-1.65597E-6/8,
  xj3      = -2.53881E-6,
  qo       = 1.0188142772142641, //1 + 120/6378.135,
  s        = 1.01222928018927161,//1 + 78/6378.135,
  e6a      = 1E-6,

  xmnpda   =  1440.0,        //{Minutes per day}
  secday   = 86400.0,        // {Seconds per day}
  omega_E  = 1.00273790934,  // {Earth rotations per sidereal day (non-constant)}
  omega_ER = 6.30038809891706375,
	     //1.00273790934*3.1415926535897932385*2,  // {Earth rotation, radians per sidereal day}

  half_sec = 0.00000578703703704, //.5/86400

  dpinit   = 1,               //Deep-space initialization code
  dpsec    = 2,               //Deep-space secular code
  dpper    = 3;               //Deep-space periodic code


unsigned int eclipsed=FALSE;
unsigned int show_vis=FALSE;

double   civil,
	 nautical,
	 astronomical;
int      iflag,ideep;
double   epoch;
double   xmo,xnodeo,omegao,eo,xincl,
	 xno,xndt2o,xndd6o,bstar,
	 julian_epoch,xke;
double   ds50;

unsigned int visible;

typedef char   clock_time[12];
typedef char   date[11];

MATRIX Solar_Pos;
double RadialSun,ARSun,DECSun;

//######################## FUNZIONI MATEMATICHE ############################

//Restituisce la somma dei prodotti
double Dot(MATRIX v1,MATRIX v2)
{
  return(v1[0][0]*v2[0][0] + v1[1][0]*v2[1][0] + v1[2][0]*v2[2][0]);
}

//Restituisce il modulo
double Magnitude(MATRIX v)
{
  return(sqrt(v[0][0]*v[0][0]+v[1][0]*v[1][0]+v[2][0]*v[2][0]));
}

void Cross(MATRIX v1,MATRIX v2,MATRIX v3)
{
  v3[0][0] = v1[1][0]*v2[2][0] - v1[2][0]*v2[1][0];
  v3[1][0] = v1[2][0]*v2[0][0] - v1[0][0]*v2[2][0];
  v3[2][0] = v1[0][0]*v2[1][0] - v1[1][0]*v2[0][0];
}


//Distanza 2 p.ti nello spazio
double Dista3D (MATRIX P1,MATRIX P2)
//{ Reference: by Diego Bonata, 1994 }
{
return(sqrt((P1[0][0]-P2[0][0])*(P1[0][0]-P2[0][0])+
	    (P1[1][0]-P2[1][0])*(P1[1][0]-P2[1][0])+
	    (P1[2][0]-P2[2][0])*(P1[2][0]-P2[2][0])));
}

//Traslazione assi in 3D
//ChangeXYZ Punto di cui cambiare le coordinate
//TralaXYZ  Coordinate della Nuova Terna rispetto alla prima
void TraslaXYZ (MATRIX ChangeXYZ,MATRIX TraslaXYZ)
{
ChangeXYZ[0][0] = ChangeXYZ[0][0] - TraslaXYZ[0][0];
ChangeXYZ[1][0] = ChangeXYZ[1][0] - TraslaXYZ[1][0];
ChangeXYZ[2][0] = ChangeXYZ[2][0] - TraslaXYZ[2][0];
}

//Definisce righe/colonne della matrice di Eulero
void SetEulero(double Chi,double Teta,double Fi,MATRIX Eulero)
//{ Reference: by Diego Bonata Appunti di Meccanica del Volo, 1995 }
{
double Cc,Sc,Ct,St,Cf,Sf;

 Cc = cos(Chi);
 Sc = sin(Chi);
 Ct = cos(Teta);
 St = sin(Teta);
 Cf = cos(Fi);
 Sf = sin(Fi);

 Eulero[0][0] =  Cf*Cc - Sf*Ct*Sc;
 Eulero[0][1] =  Cf*Sc + Sf*Ct*Cc;
 Eulero[0][2] =  Sf*St;
 Eulero[1][0] = -Sf*Cc - Cf*Ct*Sc;
 Eulero[1][1] = -Sf*Sc + Cc*Ct*Cf;
 Eulero[1][2] =  Cf*St;
 Eulero[2][0] =  St*Sc;
 Eulero[2][1] = -St*Cc;
 Eulero[2][2] =  Ct;
}

/*
//Conversione da Coord.Polari->Coord.Cartesiane
void PolarToXYZ (MATRIX XYZ,MATRIX ArDecDis)
//{ Reference: by Diego Bonata, Appunti di Geometria, 1987 }
{
double sup1,sup2;

 sup1=GtoR(ArDecDis[0][0]);
 sup2=GtoR(ArDecDis[1][0]);
 XYZ[0][0]=ArDecDis[2][0]*cos(sup1)*cos(sup2);
 XYZ[1][0]=ArDecDis[2][0]*cos(sup2)*sin(sup1);
 XYZ[2][0]=ArDecDis[2][0]*sin(sup2);
}
*/

//Conversione da Coord.Cartesiane->Coord.Polari
void XYZToPolar (MATRIX ArDecDis,MATRIX XYZ)
//{ Reference: by Diego Bonata, Appunti di Geometria, 1987 }
{
 ArDecDis[2][0]=Magnitude(XYZ);                      //Distanza (origine)
 ArDecDis[0][0]=AcTan(XYZ[1][0],XYZ[0][0]);          //AR
 ArDecDis[1][0]=pi/2-acos(XYZ[2][0]/ArDecDis[2][0]); //Dec
}


//######################## FUNZIONI DI CALENDARIO ##########################

//Restituisce il giorno giuliano dell'anno
double Julian_Date_of_Year(double year)
{
//   Astronomical Formulae for Calculators, Jean Meeus, pages 23-25
//   Calculate Julian Date of 0.0 Jan year }
long   A,B;

 year = year - 1;
 A = Trunc(year/100);
 B = 2 - A + Trunc(A/4);
 return (Trunc(365.25 * year) + Trunc(30.6001 * 14)+1720994.5+B);
}

//Restituisce il giorno giuliano dell'epoca dopo l' 1 Gennaio
double Julian_Date_of_Epoch(double epoch)
{
double year,day;
double supp;

  year = 1900 + Trunc(epoch*1E-3);
  day  = modf(epoch*1E-3,&supp)*1E3;

return (Julian_Date_of_Year(year) + day);
}

//Restituisce la frazione di giorno decimale
double Fraction_of_Day(unsigned int hr,unsigned int mi,unsigned int se,unsigned int hu)
{
 return(((double)hr + ((double)mi + ((double)se + (double)hu/100)/60)/60)/24);
}

//Restituisce il giorno dell'anno
unsigned int DOY(unsigned int yr,unsigned int mo,unsigned int dy)
{
unsigned int days[12]= {31,28,31,30,31,30,31,31,30,31,30,31};
unsigned int i,day;
double supp;

 day = 0;
 for (i=0;i<mo-1;i++) day = day + days[i];
 day = day + dy;
 if (mo>2) {
   if (modf((double)yr/4,&supp)==0) {
     if (modf((double)yr/100,&supp)!=0 || modf((double)yr/400,&supp)==0) day=day+1;
   }
 }

return day;
}

//Preleva da jd l'ora e restituisce nel formato (Stringa) HH:MM:SS (se full=1)
//                  oppure          nel formato (Stringa) HHMMSS...(se full=0)
void Time_of_Day(double jd,unsigned int full,clock_time ora)
{
long   hr,mn;
double time,sc;
double supp;
char   ctime[4],Hore[12];
int    j;

  for (j=0;j<12;j++) Hore[j]=0;
  time = modf(jd - 0.5 + half_sec,&supp);
  time = 24 * modf(jd - 0.5 + half_sec,&supp);
  hr   = Trunc(time);
  time = 60 * modf(time,&supp);
  mn   = Trunc(time);
  sc   = 60 * modf(time,&supp) - 0.5;

  if (full) {
    ltoa(hr,Hore,10);
    j=strlen(Hore);
    Hore[j]  =':';  Hore[j+1]=0;
    ltoa(mn,ctime,10);
    strcat(Hore,ctime);
    j=strlen(Hore);
    Hore[j]=':';
    Hore[j+1]=0;
    itoa((int)sc,ctime,10);
    strcat(Hore,ctime);
  }
  else{
     time=10000*hr+100*mn+sc;
     ltoa((long)time,Hore,10);
  }

 strcpy(ora,Hore);
}

//Preleva jd e restituisce la stringa contenente ANNO/MESE/GIORNO
void Calendar_Date(double jd,date data)
//  { Astronomical Formulae for Calculators, Jean Meeus, pages 26-27 }
{
long      Z,month;
double    A,B,C,D,E,F,alpha;
double    day,year;
char      syear[4];
//date      cdate;
double    supp;

 Z = Trunc(jd + 0.5);
 F = modf(jd + 0.5,&supp);
 if (Z < 2299161)    A= Z;
 else {
    alpha = Trunc((Z - 1867216.25)/36524.25);
    A = Z + 1 + alpha - Trunc(alpha/4);
 }
  B = A + 1524;
  C = Trunc((B - 122.1)/365.25);
  D = Trunc(365.25 * C);
  E = Trunc((B - D)/30.6001);
  day = B - D - Trunc(30.6001 * E) + F;
  if (E < 13.5)  month = Round(E - 1);
  else           month = Round(E - 13);
  if (month > 2.5)    year = C - 4716;
  else                year = C - 4715;

  Str(year,4,0,syear);
  strcpy(data,syear);
  ltoa(month,syear,10);
  strcat(data,"/\0");
  if (month>9) strcat(data,syear);
  else { strcat(data," \0"); strcat(data,syear);
  }
  strcat(data,"/\0");
  month=(long)(Trunc(day));
  ltoa(month,syear,10);
  strcat(data,syear);
}

//Usata da deep
double ThetaG(double epoch)
//{ Reference:  The 1992 Astronomical Almanac, page B6. }
{
double year,day,UT,jd,TU,GMST;
double supp;

  year   = 1900 + Trunc(epoch*1E-3);
  day    = modf(epoch*1E-3,&supp)*1E3;
  UT     = modf(day,&supp);
  day    = Trunc(day);
  jd     = Julian_Date_of_Year(year) + day;
  TU     = (jd - 2451545.0)/36525;
  GMST   = 24110.54841 + TU * (8640184.812866 + TU * (0.093104 - TU * 6.2E-6));
  GMST   = Modulus(GMST + secday*omega_E*UT,secday);

  ds50   = jd - 2433281.5 + UT;

return (twopi * GMST/secday);
}

double ThetaG_JD(double jd)
//{ Reference:  The 1992 Astronomical Almanac, page B6. }
{
double UT,TU,GMST;
double supp;

  UT   = modf(jd + 0.5,&supp);
  jd   = jd - UT;
  TU   = (jd - 2451545.0)/36525;
  GMST = 24110.54841 + TU * (8640184.812866 + TU * (0.093104 - TU * 6.2E-6));
  GMST = Modulus(GMST + secday*omega_E*UT,secday);

return(twopi * GMST/secday);
}

//######################## FUNZIONI SOLARI ################################

//Calcola La posizione del Sole in ECI che inserisco in Solar_Pos e RadialSun
void Cal_Solar_Pos(double time,MATRIX Solar_Vector)
//{ Reference: Jean Meeus, by Diego Bonata, 1996 }
{
double  T,M,L,e,C,O,Lsa,nu,R,eps;
double  A,B,F,G,H,I,CorrLong,CorrR;

 T   = (time - 2415020.0)/36525;

 A = 153.23 + 22518.7541 * T;
 B = 216.57 + 45037.5082 * T;
 F = 312.69 + 32964.3577 * T;
 G = 350.74 + 445267.1142* T - 0.00144 * T*T;
 H = 231.19 + 20.20      * T;
 I = 353.40 + 65928.7155 * T;
 CorrLong =   0.00134*cos(A) + 0.00154*cos(B) + 0.00200*cos(F)
	    + 0.00179*sin(G) + 0.00178*sin(H);
 CorrR    =   0.00000543*sin(A) + 0.00001575*sin(B) + 0.00001627*sin(F)
	    + 0.00003076*cos(G) + 0.00000927*sin(I);

 M   = GtoR(Modulus(358.47583 + Modulus(35999.04975*T,360.0)
	- (0.000150 + 0.0000033*T)*T*T,360.0));
 L   = GtoR(Modulus(279.69668 + Modulus(36000.76892*T,360.0)
	+ 0.0003025*T*T,360.0));
 e   = 0.01675104 - (0.0000418 + 0.000000126*T)*T;
 C   = GtoR((1.919460 - (0.004789 + 0.000014*T)*T)*sin(M)
	+ (0.020094 - 0.000100*T)*sin(2*M) + 0.000293*sin(3*M));
 O   = GtoR(Modulus(259.18 - 1934.142*T,360.0));
 Lsa = Modulus(L + C - GtoR(0.00569 - 0.00479*sin(O)+CorrLong),twopi);
 eps = GtoR(23.452294 - (0.0130125 + (0.00000164 - 0.000000503*T)*T)*T
	+ 0.00256*cos(O));
 ARSun = RtoG(AcTan(cos(eps)*sin(Lsa),cos(Lsa)))/15;
 DECSun= RtoG(asin(sin(eps)*sin(Lsa)));
 nu  = Modulus(M + C,twopi);
 R   = 1.0000002*(1 - e*e)/(1 + e*cos(nu));
 R   = AU*(R+CorrR);
 Solar_Vector[0][0] = cos(Lsa);
 Solar_Vector[1][0] = sin(Lsa)*cos(eps);
 Solar_Vector[2][0] = sin(Lsa)*sin(eps);
// printf("\n#1=%.12f 2=%.12f 3=%.12f  R=%.6f",Solar_Vector[0][0],Solar_Vector[1][0],Solar_Vector[2][0],R);
 RadialSun          = R;
}

//Visibilita' di un satellite SAT1=Osservatore
void Eclipse_Sat_Sun(MATRIX Sat1)
//{ Reference: by Diego Bonata, 1996 }
{
double r1,r2,r12,Dso,TO,x,CO,theta;

// Cal_Solar_Pos(time,Solar_Pos); gia' nel programma
 r2  = Solar_Pos[0][0]*Solar_Pos[0][0]+Solar_Pos[1][0]*Solar_Pos[1][0]
       + Solar_Pos[2][0]*Solar_Pos[2][0];
 r1  = Sat1[0][0]*Sat1[0][0]+Sat1[1][0]*Sat1[1][0]+Sat1[2][0]*Sat1[2][0];
 r12 = Dot (Sat1,Solar_Pos);
 theta = acos(r12/sqrt(r1*r2));
 if (theta>pi/2) {
    Dso = sqrt(r1-r12*r12/r2);
    TO  = sqrt(r1-Dso*Dso);
    x   = xkmper*RadialSun/(sr-xkmper);
    CO  = (x - TO)*xkmper/sqrt(x*x-xkmper*xkmper);
    if (Dso < CO) eclipsed = TRUE;
    else          eclipsed = FALSE;
 }
 else eclipsed = FALSE;

}

//Visibilita' di una stella SAT1=Osservatore SAT2=Stella
void Eclipse_Sat_Star(MATRIX Sat1, MATRIX Sat2)
//{ Reference: by Diego Bonata, 1996 }
{
double r1,r2,r12,gam;
//NB ne r1 ne r2 possono essere mai =0 Altrimenti saremmo dentro la terra
 r2   = Sat2[0][0]*Sat2[0][0]+Sat2[1][0]*Sat2[1][0]+Sat2[2][0]*Sat2[2][0];
 r1   = Sat1[0][0]*Sat1[0][0]+Sat1[1][0]*Sat1[1][0]+Sat1[2][0]*Sat1[2][0];
 r12  = Dot (Sat1,Sat2);
 gam  = acos(r12/sqrt(r1*r2));
 if (gam<pi/2) eclipsed = FALSE;
 else {
    if (sqrt(r1)*sin(gam)<=xkmper) eclipsed = TRUE;
    else                           eclipsed = FALSE;
 }
}

//Visibilita' di un satellite SAT1=Osservatore SAT2=Osservato
void Eclipse_Sat1_Sat2(MATRIX Sat1, MATRIX Sat2)
//{ Reference: by Diego Bonata, 1996 }
{
double r1,r2,r12,t,h[3],sxM,sxm,syM,sym,szM,szm;
int    Ecl;

 r1  = Sat1[0][0]*Sat1[0][0]+Sat1[1][0]*Sat1[1][0]+Sat1[2][0]*Sat1[2][0];
 r2  = Sat2[0][0]*Sat2[0][0]+Sat2[1][0]*Sat2[1][0]+Sat2[2][0]*Sat2[2][0];
 r12 = Dot (Sat1,Sat2);
 t   = (r1 - r12)/(r2 + r1 - 2*r12);
 h[0]= Sat1[0][0]+(Sat2[0][0]-Sat1[0][0])*t;
 h[1]= Sat1[1][0]+(Sat2[1][0]-Sat1[1][0])*t;
 h[2]= Sat1[2][0]+(Sat2[2][0]-Sat1[2][0])*t;
 if (sqrt(h[0]*h[0]+h[1]*h[1]+h[2]*h[2])<xkmper) {
     if (Sat1[0][0]>Sat2[0][0]) { sxM=Sat1[0][0]; sxm=Sat2[0][0]; }
     else                       { sxM=Sat2[0][0]; sxm=Sat1[0][0]; }
     if (Sat1[1][0]>Sat2[1][0]) { syM=Sat1[1][0]; sym=Sat2[1][0]; }
     else                       { syM=Sat2[1][0]; sym=Sat1[1][0]; }
     if (Sat1[2][0]>Sat2[2][0]) { szM=Sat1[2][0]; szm=Sat2[2][0]; }
     else                       { szM=Sat2[2][0]; szm=Sat1[2][0]; }
     if ((h[0]>sxm && h[0]<sxM) && (h[1]>sym && h[1]<syM) && (h[2]>szm && h[2]<szM))
	  eclipsed = TRUE;
     else eclipsed = FALSE;
 }
 else eclipsed = FALSE;
}

//############## FUNZIONI DI CONVERSIONE DI COORDINATE #######################
void Calculate_User_PosVel(MATRIX geodetic,double time,
			   MATRIX obs_pos,MATRIX obs_vel)
{
//{ Reference:  The 1992 Astronomical Almanac, page K11. }
double mfactor = twopi*omega_E/secday;
double lat,lon,alt,theta,c,s,achcp;

  lat   = geodetic[0][0];
  lon   = geodetic[1][0];
  alt   = geodetic[2][0];
  theta = Modulus(ThetaG_JD(time) + lon,twopi);
  c     = 1/sqrt(1 + f*(f - 2)*sin(lat)*sin(lat));
  s     = (1 - f)*(1 - f)*c;
  achcp = (xkmper*c + alt)*cos(lat);
  obs_pos[0][0] = achcp*cos(theta);            // {kilometers}
  obs_pos[1][0] = achcp*sin(theta);
  obs_pos[2][0] = (xkmper*s + alt)*sin(lat);
  obs_vel[0][0] = -mfactor*obs_pos[1][0];      // {kilometers/second}
  obs_vel[1][0] =  mfactor*obs_pos[0][0];
  obs_vel[2][0] =  0;

}

void Calculate_LatLonAlt(MATRIX pos,double time,MATRIX geodetic)
{
//{ Reference:  The 1992 Astronomical Almanac, page K12. }
double  lat,lon,alt,theta,r,e2,phi,c;

  theta  = AcTan(pos[1][0],pos[0][0]);
  lon    = Modulus(theta - ThetaG_JD(time),twopi);
  r      = sqrt((pos[0][0])*(pos[0][0]) + (pos[1][0])*(pos[1][0]));
  e2     = f*(2 - f);
  lat    = AcTan(pos[2][0],r);
  do {
     phi    = lat;
     c      = 1/sqrt(1 - e2*sin(phi)*sin(phi));
     lat    = AcTan(pos[2][0] + xkmper*c*e2*sin(phi),r);
  } while(fabs(lat - phi) >= 1E-10);
  alt         = r/cos(lat) - xkmper*c;
  geodetic[0][0] = lat;  // {radians}
  geodetic[1][0] = lon;  // {radians}
  geodetic[2][0] = alt;  // {kilometers}
}

void Calculate_Obs(MATRIX pos,MATRIX vel,MATRIX geodetic,double time,
		   MATRIX obs_set)
{
int    i;
double lat,lon,alt,theta,sin_lat,cos_lat,sin_theta,cos_theta,
       el,azim,top_s,top_e,top_z,rg;
MATRIX obs_pos,obs_vel;
MATRIX range,rgvel;

  obs_pos = mat_creat(3,1,ZERO_MATRIX);
  obs_vel = mat_creat(3,1,ZERO_MATRIX);
  range   = mat_creat(3,1,ZERO_MATRIX);
  rgvel   = mat_creat(3,1,ZERO_MATRIX);

  Calculate_User_PosVel(geodetic,time,obs_pos,obs_vel);
  for (i=0;i<3;i++) {
      range[i][0] = pos[i][0] - obs_pos[i][0];
      rgvel[i][0] = vel[i][0] - obs_vel[i][0];
  }
  rg     = Magnitude(range);
  lat    = geodetic[0][0];
  lon    = geodetic[1][0];
  //alt    = geodetic[2][0];
  theta  = theta = Modulus(ThetaG_JD(time) + lon,twopi);
  sin_lat= sin(lat);
  cos_lat= cos(lat);
  sin_theta = sin(theta);
  cos_theta = cos(theta);
  top_s     = sin_lat*cos_theta*range[0][0]
	      + sin_lat*sin_theta*range[1][0]
	      - cos_lat*range[2][0];
  top_e     = - sin_theta*range[0][0]
	      + cos_theta*range[1][0];
  top_z     = cos_lat*cos_theta*range[0][0]
	      + cos_lat*sin_theta*range[1][0]
	      + sin_lat*range[2][0];
  azim      = atan(-top_e/top_s);              //  {Azimuth}
  if (top_s > 0)     azim = azim + pi;
  if (azim < 0)      azim = azim + twopi;
  el  = asin(top_z/rg);
  obs_set[0][0] = azim;                        // {Azimuth (radians)}
  obs_set[1][0] = el;                          // {Elevation (radians)}
  obs_set[2][0] = rg;                          // {Range (kilometers)}
//obs_set[3][0] = Dot(range,rgvel)/range[3][0];// {Range Rate (kilometers/second)}

//{ Corrections for atmospheric refraction }
//{ Reference:  Astronomical Algorithms by Jean Meeus, pp. 101-104 }
//{ Note:  Correction is meaningless when apparent elevation is below horizon }
  obs_set[1][0] = obs_set[1][0] + GtoR((1.02/tan(GtoR(RtoG(el)+10.3/(RtoG(el)+5.11))))/60);
  if (obs_set[1][0] >= 0)    visible = TRUE;
  else {
     obs_set[1][0]  = el; // {Reset to true elevation}
     visible        = FALSE;
  }//    end; {else}

 mat_free(obs_pos);
 mat_free(obs_vel);
 mat_free(range);
 mat_free(rgvel);
}
//end; {Procedure Calculate_Obs}

void Calculate_RADec(MATRIX pos,MATRIX vel,MATRIX geodetic,double time,
		     MATRIX obs_set)
//{ Reference:  Methods of Orbit Determination by Pedro Ramon Escobal, pp. 401-402}
{
double   phi,theta,sin_theta,cos_theta,sin_phi,cos_phi,az,el,Lxh,Lyh,Lzh,
	 Sx,Ex,Zx,Sy,Ey,Zy,Sz,Ez,Zz,Lx,Ly,Lz,cos_delta,sin_alpha,cos_alpha;

  Calculate_Obs(pos,vel,geodetic,time,obs_set);
  if (visible) {
     az    = obs_set[0][0];
     el    = obs_set[1][0];
     phi   = geodetic[0][0];
     theta = Modulus(ThetaG_JD(time) + geodetic[1][0],twopi);
     sin_theta = sin(theta);
     cos_theta = cos(theta);
     sin_phi   = sin(phi);
     cos_phi   = cos(phi);
     Lxh   = -cos(az)*cos(el);
     Lyh   =  sin(az)*cos(el);
     Lzh   =  sin(el);
     Sx    = sin_phi*cos_theta;
     Ex    = -sin_theta;
     Zx    = cos_theta*cos_phi;
     Sy    = sin_phi*sin_theta;
     Ey    = cos_theta;
     Zy    = sin_theta*cos_phi;
     Sz    = -cos_phi;
     Ez    = 0;
     Zz    = sin_phi;
     Lx    = Sx*Lxh + Ex*Lyh + Zx*Lzh;
     Ly    = Sy*Lxh + Ey*Lyh + Zy*Lzh;
     Lz    = Sz*Lxh + Ez*Lyh + Zz*Lzh;
     obs_set[1][0] = asin(Lz);                // {Declination (radians)}
     cos_delta     = sqrt(1 - Lz*Lz);
     sin_alpha     = Ly/cos_delta;
     cos_alpha     = Lx/cos_delta;
     obs_set[0][0] = AcTan(sin_alpha,cos_alpha);// {Right Ascension (radians)}
     obs_set[0][0] = Modulus(obs_set[0][0],twopi);
  }//  end; {if}
}//  end; {Procedure Calculate_RADec}

