#include "math.h"

#include "matrix.h"
#include "miemath.h"


#define TRUE  1      /*  logical operators  */
#define FALSE 0


extern unsigned int eclipsed;
extern unsigned int show_vis;

extern double   civil,
		nautical,
		astronomical;
extern int      iflag,ideep;
extern double   epoch;
extern double   xmo,xnodeo,omegao,eo,xincl,
		xno,xndt2o,xndd6o,bstar,
		julian_epoch,xke;
extern double   ds50;

typedef char    clock_time[12],date[11];

extern MATRIX   Solar_Pos;
extern double   RadialSun,ARSun,DECSun;

extern double XNode,XIncl;

extern const double
  pi,twopi,sr,AU,ae,tothrd,xkmper,
  f,ge,J2,J3,J4,ck2,ck4,xj3,qo,s,
  e6a,xmnpda,secday,omega_E,omega_ER,
  half_sec,

//PROGRAMMA
  dpinit,dpsec,dpper;               //Deep-space periodic code

//################## FUNZIONI DI CALCOLO SATELLITARE #######################

//DEFINITE in SGP4SDP4
//Procedura generale che seleziona da sola il modello
extern void SGP (double time,  MATRIX Pos, MATRIX Vel);
//Modello SGP4 Per satelli tipo Near Earth
extern void SGP4(double tsince,int *iflag, MATRIX Pos, MATRIX Vel);
//Modello SDP4 Per satelli tipo Deep Space Orbit
extern void SDP4(double tsince,int *iflag, MATRIX Pos, MATRIX Vel);
//Definizione Cost. Iniziali
extern void Define_Derived_Constants(void);

//################## FUNZIONI DI CALCOLO SATELLITARE #######################

//Date le Coord. Latitudine,Longitudine,Altitudine calcola le coord.ECI
extern void Calculate_User_PosVel(MATRIX geodetic, double time,
			   MATRIX obs_pos,  MATRIX obs_vel);

//Date le Coord.ECI Calcola Latitudine,Longitudine,Altitudine
extern void Calculate_LatLonAlt(MATRIX pos, double time, MATRIX geodetic);

//Date le Coord.ECI Calcola Azimut e Altezza sull'Orizzonte
//ATTENZIONE: la longitudine va passata non con Est(+) Ovest(-) come normale
//            ma viceversa (Log. contenuta in geodetic[2][0])
extern void Calculate_Obs(MATRIX pos,MATRIX vel,MATRIX geodetic,double time,
		   MATRIX obs_set);

//Date le Coord.ECI Calcola Ascensione Retta e Declinazione
//ATTENZIONE: la longitudine va passata non con Est(+) Ovest(-) come normale
//            ma viceversa (Log. contenuta in geodetic[2][0])
extern void Calculate_RADec(MATRIX pos,MATRIX vel,MATRIX geodetic,double time,
		     MATRIX obs_set);

//######################## FUNZIONI MATEMATICHE ############################

//Restituisce la somma dei prodotti
extern double Dot(MATRIX v1,MATRIX v2);

//Restituisce il modulo
extern double Magnitude(MATRIX v);

extern void   Cross(MATRIX v1,MATRIX v2,MATRIX v3);

//Restituisce il giorno dell'anno
extern unsigned int DOY(unsigned int yr,unsigned int mo,unsigned int dy);

//Distanza 2 p.ti nello spazio
extern double Dista3D (MATRIX P1,MATRIX P2);

//Traslazione assi in 3D
extern void TraslaXYZ (MATRIX ChangeXYZ,MATRIX TraslaXYZ);

//Definisce righe/colonne della matrice di Eulero
extern void SetEulero(double Chi,double Teta,double Fi,MATRIX Eulero);

//Trasforma da Coordinate Cartesiane a Coordinate Polari
extern void XYZToPolar (MATRIX ArDecDis,MATRIX XYZ);

//######################## FUNZIONI DI CALENDARIO ##########################

//Preleva jd e restituisce la stringa contenente ANNO/MESE/GIORNO
extern void   Calendar_Date(double jd,date data);

//Preleva da jd l'ora e restituisce nel formato (Stringa) HH:MM:SS (se full=1)
//                  oppure          nel formato (Stringa) HHMMSS...(se full=0)
extern void   Time_of_Day(double jd,unsigned int full,clock_time ora);

//Restituisce il giorno giuliano dell'anno
extern double Julian_Date_of_Year(double year);

//Restituisce il giorno giuliano dell'anno
extern double Julian_Date_of_Epoch(double epoch);

//di servizio
extern double ThetaG(double epoch);

//di servizio
extern double ThetaG_JD(double jd);

//Restituisce l'equazione del tempo
extern double Delta_ET(double year);

//Restituisce la frazione di giorno decimale
extern double Fraction_of_Day(unsigned int hr,unsigned int mi,
		       unsigned int se,unsigned int hu);

//######################## FUNZIONI SOLARI ##########################

//Visibilita' di un satellite SAT1=Osservatore SAT2=Osservato
extern void   Eclipse_Sat1_Sat2(MATRIX Sat1, MATRIX Sat2);

//Visibilita' di una stella SAT1=Osservatore SAT2=Stella
extern void   Eclipse_Sat_Star(MATRIX Sat1, MATRIX Sat2);

//Visibilita' di un satellite SAT1=Osservatore SAT2=Sun
extern void   Eclipse_Sat_Sun(MATRIX Sat1);

//Calcola La posizione del Sole in ECI che inserisco in Solar_Pos e RadialSun
extern void   Cal_Solar_Pos(double time,MATRIX Solar_Vector);
